@@ -1,13 +1,18 @@
+import logging
 import sys
 
-from warnings import warn
-
 from datetime import datetime
 from dateutil import (parser, tz)
 
 
+#: Logger for core module
+LOGGER = logging.getLogger('github2.core')
+
 #: Running under Python 3
-PY3K = sys.version_info[0] == 3 and True or False
+PY3K = sys.version_info[0] == 3
+
+#: Running under Python 2.7, or newer
+PY27 = sys.version_info[:2] >= (2, 7)
 
 GITHUB_DATE_FORMAT = "%Y/%m/%d %H:%M:%S %z"
 # We need to manually mangle the timezone for commit date formatting because it
@@ -100,16 +105,18 @@ def requires_auth(f):
     :param func f: Function to wrap
     :raises AuthError: If function called without an authenticated session
     """
-    # When Python 2.4 support is dropped move straight to functools.wraps, don't
-    # pass go and don't collect $200.
+    # When Python 2.4 support is dropped move straight to functools.wraps,
+    # don't pass go and don't collect $200.
     def wrapper(self, *args, **kwargs):
         if not self.request.access_token and not self.request.api_token:
             raise AuthError("%r requires an authenticated session"
                             % f.__name__)
         return f(self, *args, **kwargs)
     wrapped = wrapper
+    wrapped.__orig_func__ = f
     wrapped.__name__ = f.__name__
     wrapped.__doc__ = f.__doc__ + """\n.. warning:: Requires authentication"""
+    wrapped.__module__ = f.__module__
     wrapped.requires_auth = True
     return wrapped
 
@@ -130,19 +137,37 @@ def enhanced_by_auth(f):
 class GithubCommand(object):
 
     def __init__(self, request):
+        """Main API binding interface
+
+        :param github2.request.GithubRequest request: HTTP request handler
+        """
         self.request = request
 
     def make_request(self, command, *args, **kwargs):
+        """Make an API request
+
+        Various options are supported if they exist in ``kwargs``:
+
+        * The value of a ``method`` argument will define the HTTP method
+          to perform for this request, the default is ``GET``
+        * The value of a ``filter`` argument will restrict the response to that
+          data
+        * The value of a ``page`` argument will be used to fetch a specific
+          page of results, default of 1 is assumed if not given
+        """
         filter = kwargs.get("filter")
         post_data = kwargs.get("post_data") or {}
-        method = kwargs.get("method", "GET")
-        if method.upper() == "POST" or method.upper() == "GET" and post_data:
+        page = kwargs.pop("page", 1)
+        if page and not page == 1:
+            post_data["page"] = page
+        method = kwargs.get("method", "GET").upper()
+        if method == "POST" or method == "GET" and post_data:
             response = self.request.post(self.domain, command, *args,
                                          **post_data)
-        elif method.upper() == "PUT":
+        elif method == "PUT":
             response = self.request.put(self.domain, command, *args,
                                         **post_data)
-        elif method.upper() == "DELETE":
+        elif method == "DELETE":
             response = self.request.delete(self.domain, command, *args,
                                            **post_data)
         else:
@@ -152,23 +177,40 @@ class GithubCommand(object):
         return response
 
     def get_value(self, *args, **kwargs):
+        """Process a single-value response from the API
+
+        If a ``datatype`` parameter is given it defines the
+        :class:`BaseData`-derived class we should build from the provided data
+        """
         datatype = kwargs.pop("datatype", None)
         value = self.make_request(*args, **kwargs)
         if datatype:
-            # unicode keys are not accepted as kwargs by python, see:
-            #http://mail-archives.apache.org/mod_mbox/qpid-dev/200609.mbox/%3C1159389941.4505.10.camel@localhost.localdomain%3E
-            # So we make a local dict with the same keys but as strings:
-            return datatype(**dict((str(k), v) for (k, v) in value.iteritems()))
+            if not PY27:
+                # unicode keys are not accepted as kwargs by python, until 2.7:
+                # http://bugs.python.org/issue2646
+                # So we make a local dict with the same keys but as strings:
+                return datatype(**dict((str(k), v)
+                                       for (k, v) in value.items()))
+            else:
+                return datatype(**value)
         return value
 
     def get_values(self, *args, **kwargs):
+        """Process a multi-value response from the API
+
+        :see: :meth:`get_value`
+        """
         datatype = kwargs.pop("datatype", None)
         values = self.make_request(*args, **kwargs)
         if datatype:
-            # Same as above, unicode keys will blow up in **args, so we need to
-            # create a new 'values' dict with string keys
-            return [datatype(**dict((str(k), v) for (k, v) in value.iteritems()))
-                    for value in values]
+            if not PY27:
+                # Same as above, unicode keys will blow up in **args, so we
+                # need to create a new 'values' dict with string keys
+                return [datatype(**dict((str(k), v)
+                                        for (k, v) in value.items()))
+                        for value in values]
+            else:
+                return [datatype(**value) for value in values]
         else:
             return values
 
@@ -190,8 +232,11 @@ def doc_generator(docstring, attributes):
 
 
 class Attribute(object):
-
     def __init__(self, help):
+        """Generic object attribute for use with :class:`BaseData`
+
+        :param str help: Attribute description
+        """
         self.help = help
 
     def to_python(self, value):
@@ -210,6 +255,11 @@ class DateAttribute(Attribute):
     }
 
     def __init__(self, *args, **kwargs):
+        """Date handling attribute for use with :class:`BaseData`
+
+        :param str format: The date format to support, see
+            :data:`convertor_for_format` for supported options
+        """
         self.format = kwargs.pop("format", self.format)
         super(DateAttribute, self).__init__(*args, **kwargs)
 
@@ -230,15 +280,14 @@ class BaseDataType(type):
         super_new = super(BaseDataType, cls).__new__
 
         _meta = dict([(attr_name, attr_value)
-                        for attr_name, attr_value in attrs.items()
-                            if isinstance(attr_value, Attribute)])
+                      for attr_name, attr_value in attrs.items()
+                      if isinstance(attr_value, Attribute)])
         attrs["_meta"] = _meta
         attributes = _meta.keys()
-        attrs.update(dict([(attr_name, None)
-                        for attr_name in attributes]))
+        attrs.update(dict([(attr_name, None) for attr_name in attributes]))
 
         def _contribute_method(name, func):
-            func.func_name = name
+            func.__name__ = name
             attrs[name] = func
 
         def constructor(self, **kwargs):
@@ -260,17 +309,24 @@ class BaseDataType(type):
         return result_cls
 
 
-class BaseData(object):
-    __metaclass__ = BaseDataType
+# Ugly base class definition for Python 2 and 3 compatibility, where metaclass
+# syntax is incompatible
+class BaseData(BaseDataType('BaseData', (object, ), {})):
+    """Wrapper for API responses
 
+    .. warning::
+       Supports subscript attribute access purely for backwards compatibility,
+       you shouldn't rely on that functionality in new code
+    """
     def __getitem__(self, key):
         """Access objects's attribute using subscript notation
 
         This is here purely to maintain compatibility when switching ``dict``
         responses to ``BaseData`` derived objects.
         """
-        warn("Subscript access on %r is deprecated, use object attributes"
-             % self.__class__.__name__, DeprecationWarning)
+        LOGGER.warning("Subscript access on %r is deprecated, use object "
+                       "attributes" % self.__class__.__name__,
+                       DeprecationWarning)
         if not key in self._meta.keys():
             raise KeyError(key)
         return getattr(self, key)
@@ -278,10 +334,11 @@ class BaseData(object):
     def __setitem__(self, key, value):
         """Update object's attribute using subscript notation
 
-        :see: ``BaseData.__getitem__``
+        :see: :meth:`BaseData.__getitem__`
         """
-        warn("Subscript access on %r is deprecated, use object attributes"
-             % self.__class__.__name__, DeprecationWarning)
+        LOGGER.warning("Subscript access on %r is deprecated, use object "
+                       "attributes" % self.__class__.__name__,
+                       DeprecationWarning)
         if not key in self._meta.keys():
             raise KeyError(key)
         setattr(self, key, value)
